from setuptools import setup

setup(
    name = 'FINITO_TOOLBOX',
    version = '2022.1',
    author = 'Prof. Wanderlei M. Pereira Junior, Prof. Davidson de Oliveira França Junior, Prof. Marcos Rabelo Napoleão, Civil Eng. Gabriel B. Carvalho, Civil Eng. José Vitor Carvalho and Civil Eng. Murilo Carneiro Rodrigues',
    author_email = 'wanderlei_junior@ufcat.edu.br',
    url='https://wmpjrufg.github.io/FINITO_TOOLBOX/',
    packages = ['FINITO_TOOLBOX']
)